unit AnsiStringConvObj;
(*
Usage

The code does a global replace on .pas and .dpr files
"String" to "AnsiString"
"Char" to "AnsiChar"
"PChar" to "PAnsiChar"
"Chr" to"Chr"    Chr(78) stays as 8bit ???

It Excludes words in comments or quotes
It Excludes Short Strings (String[???]) but does not check for the compiler switch  {$H-}

It will do
a single file,
a single directory or
a directory tree (Directory and all Sub directories)

In the directory mode and Directory tree mode you can flag "Edit the existing File"
 A back up is made by first renamimg the current file(s)

In Directory Tree Mode and if  NOT "Edit the existing File" you can specify
Copy all Delphi files. Seemed a good idea at the time but it is probably better
to do your backup and then process files inplace. Recreating the Tree with all Delphi
files allowed BeyondCompare to show exactly what was done but I found in many cases
I needed help, database and test data files manually copied across.

You can Also Specify Delete Existing File(s) so you can repeat the process without manually deleting.


***************************************
The INI File  and Function Conversions

If  the AProcessFunctions flag is set the software will look at the ini file
and make the flagged conversions
Eg.

[CONVERSIONS]
  CopyFile 	= CopyFileA
  CreateProcess =  CreateProcessA
  OpenService 	= OpenServiceA

*)


interface
uses Windows, SysUtils, classes, IniFiles;

type
  TAnsiStringProcessObject = class(TObject)
  private
    FInputFileData: AnsiString;
    FOutputFile: TFileStream;
    FFilePos, FFileEnd: Int64;
    FArrayOfFunctions, FArrayOfFunctionResults: array of AnsiString;
    FProcessFunctions:Boolean;
    function NextData: AnsiChar;
    function TestFollowingGap(var ALastTst: AnsiChar; ATst: AnsiChar): AnsiString;
    function TestNoGap(var ALastTst: AnsiChar; ATst: AnsiChar): AnsiString;
    function TestForString(var ARtnLastChar: AnsiChar; ATest: AnsiChar): AnsiString;
    function TestForChar(var ARtnLastChar: AnsiChar; ATest: AnsiChar): AnsiString;
    function TestForPChar(var ARtnLastChar: AnsiChar; ATest: AnsiChar): AnsiString;
    function TestForSlashComment(var ARtnLastChar: AnsiChar; ATest: AnsiChar): AnsiString;
    function TestForBrktStarComment(var ARtnLastChar: AnsiChar; ATest: AnsiChar): AnsiString;
    function TestForCrlyBrktComment(var ARtnLastChar: AnsiChar; ATest: AnsiChar): AnsiString;
    function TestForQuotedString(var ARtnLastChar: AnsiChar; ATest: AnsiChar): AnsiString;
    function TestForGeneralString(var ARtnLastChar: AnsiChar; const ACurrentText: AnsiString): AnsiString;
    procedure WriteOutput(AData: AnsiString; var ASizeToWrite: integer);
    function ProcessFile(AInputFileName, AOutputFileName: AnsiString; AProcessFunctions:Boolean): Boolean;
    function ProcessDirectorys(AInputDirName, AOutputDirectoryName: AnsiString;
      ADeleteExisting: Boolean; ADoSub: Boolean; ACopyOtherFiles: Boolean;
      ADoInplaceEdit: Boolean ; AProcessFunctions:Boolean): Boolean;
    function NextAvailableBackupName(ACurrentFileName: AnsiString): AnsiString;
    procedure SetUpDataArray;
    procedure AddFilesToList(AList: TStrings; AFilter: AnsiString; AFlag: Integer);
  public
    constructor Create;
    destructor Destroy; override;
    procedure ListOfDelphiSourceFilesToProcess(AList: TStrings; AInputDirName: AnsiString; ADoSub, AIncludeNonProcess: Boolean);
    function ProcessOneDirectory(AInputDirName, AOutputDirectoryName: AnsiString;
      ADeleteAnyExistingOutput: Boolean = false;
      ADoInplaceEdit: Boolean = false;
      AProcessFunctions:Boolean=true): Boolean;
    function ProcessAndCopyDirectoryTree(AInputDirName, AOutputDirectoryName: AnsiString;
      ADeleteAnyExistingOutput: Boolean = false; ACopyOtherFile: Boolean = false;
      ADoInplaceEdit: Boolean = false;AProcessFunctions:Boolean=true): Boolean;
    function ProcessOneFile(AInputFileName, AOutputDirectoryName: AnsiString;
      ADeleteAnyExistingOutput: Boolean = false; AProcessFunctions:Boolean=true): Boolean;
  end;

function WorkObj: TAnsiStringProcessObject;

implementation


var
  OnlyProcessObject: TAnsiStringProcessObject = nil;
const
  cAnsiResultDir = '\AnsiResult';

function WorkObj: TAnsiStringProcessObject;
begin
  if OnlyProcessObject = nil then
    TAnsiStringProcessObject.Create;
  Result := OnlyProcessObject;
end;


const
  SetOfTestChars = ['S', 's', 'C', 'c', 'P', 'p'];
  SetOfAltChars = ['/', '(', '{', ''''];
  SetOfNonGapChars = ['a'..'z', 'A'..'Z', '_', '0'..'9'];

{ TAnsiStringProcessObject }

procedure TAnsiStringProcessObject.AddFilesToList(AList: TStrings;
  AFilter: AnsiString; AFlag: Integer);

var
  SearchRec: TSearchRec;
  Path: AnsiString;
begin
  if AList = nil then exit;

  Path := ExtractFilePath(AFilter);
  if FindFirst(AFilter, 0, SearchRec) = 0 then
  try
    if AFlag = 0 then
      Alist.Add(Path + SearchRec.Name)
    else
      Alist.AddObject(Path + SearchRec.Name, TObject(AFlag));
    while FindNext(SearchRec) = 0 do
      if AFlag = 0 then
        Alist.Add(Path + SearchRec.Name)
      else
        Alist.AddObject(Path + SearchRec.Name, TObject(AFlag));
  finally
    FindClose(SearchRec);
  end;
end;

constructor TAnsiStringProcessObject.Create;
begin
  if OnlyProcessObject <> nil then
    raise Exception.Create('OnlyProcessObject<>nil in Create');
  OnlyProcessObject := self;
  inherited Create;
end;

destructor TAnsiStringProcessObject.Destroy;
begin
  FreeAndNil(FOutputFile);
  inherited;
end;

procedure TAnsiStringProcessObject.ListOfDelphiSourceFilesToProcess(
  AList: TStrings; AInputDirName: AnsiString; ADoSub, AIncludeNonProcess: Boolean);
var
  SearchRec: TSearchRec;
  DirPath: AnsiString;
begin
  if AInputDirName = '' then Exit;
  if AList = nil then exit;
  if not DirectoryExists(AInputDirName) then Exit;
  AddFilesToList(AList, AInputDirName + '\*.pas', 0);
  AddFilesToList(AList, AInputDirName + '\*.dpr', 0);
  if AIncludeNonProcess then
  begin
    AddFilesToList(AList, AInputDirName + '\*.res', 1);
    AddFilesToList(AList, AInputDirName + '\*.dfm', 2);
    AddFilesToList(AList, AInputDirName + '\*.dsk', 3);
    AddFilesToList(AList, AInputDirName + '\*.dsk', 4);
    AddFilesToList(AList, AInputDirName + '\*.dof', 5);
    AddFilesToList(AList, AInputDirName + '\*.dpk', 6);
    AddFilesToList(AList, AInputDirName + '\*.ddp', 7);
    AddFilesToList(AList, AInputDirName + '\*.cfg', 8);
    AddFilesToList(AList, AInputDirName + '\*.bds*', 9);
    AddFilesToList(AList, AInputDirName + '\*.inc', 10);
    AddFilesToList(AList, AInputDirName + '\*.obj', 11);
  end;
  if ADoSub and (AList.Count < 10000) then
  begin
    if FindFirst(AInputDirName + '\*', faDirectory, SearchRec) = 0 then
    try
      DirPath := AInputDirName + '\' + SearchRec.Name;
      while DirPath <> '' do
      begin
        if SearchRec.Name[1] <> '.' then
          if DirectoryExists(DirPath) then
            ListOfDelphiSourceFilesToProcess(Alist, DirPath, true, AIncludeNonProcess);
        if FindNext(SearchRec) = 0 then
          DirPath := AInputDirName + '\' + SearchRec.Name
        else
          DirPath := '';
      end;
    finally
      FindClose(SearchRec);
    end;
  end;
end;

function TAnsiStringProcessObject.NextAvailableBackupName(
  ACurrentFileName: AnsiString): AnsiString;
var
  i: integer;
  Exten, NewExten: AnsiString;
begin
  i := 1;
  Exten := ExtractFileExt(ACurrentFileName);
  case Length(Exten) of
    0, 1: NewExten := '_bk' + IntToStr(i) + '.~';
    2: NewExten := '_bk' + IntToStr(i) + '.~' + Exten[2];
  else NewExten := '_bk' + IntToStr(i) + '.~' + Exten[2] + Exten[3];
  end;

  Result := ChangeFileExt(ACurrentFileName, NewExten);
  while FileExists(Result) do
  begin
    Result := StringReplace(Result, '_bk' + IntToStr(i), '_bk' + inttostr(i + 1), []);
    inc(i);
  end;
end;

function TAnsiStringProcessObject.NextData: AnsiChar;
begin
  Result := #0;
  if FFilePos > FFileEnd then Exit;
  Result := FInputFileData[FFilePos];
  Inc(FFilePos);
  if Ord(Result) > 127 then Result := Chr(Ord(Result) - 128); //Move all to 0 ..128
end;

function TAnsiStringProcessObject.ProcessAndCopyDirectoryTree(
  AInputDirName, AOutputDirectoryName: AnsiString;
  ADeleteAnyExistingOutput, ACopyOtherFile: Boolean;
  ADoInplaceEdit: Boolean;AProcessFunctions:Boolean): Boolean;
var
  InputDirectoryExpanded, OutputDirRoot: AnsiString;

begin
  if AOutputDirectoryName = '' then
    raise Exception.Create('You must provide the root output directory');
  InputDirectoryExpanded := Lowercase(ExpandFileName(AInputDirName));
  OutputDirRoot := Lowercase(ExpandFileName(AOutputDirectoryName)) + '\';
  if Pos(InputDirectoryExpanded, OutputDirRoot) = 1 then
    raise Exception.Create('Root output directory cannot exist under origin');

  Result := ProcessDirectorys(AInputDirName, AOutputDirectoryName,
    ADeleteAnyExistingOutput, true, ACopyOtherFile, ADoInplaceEdit,AProcessFunctions);
end;

function TAnsiStringProcessObject.ProcessDirectorys(AInputDirName,
  AOutputDirectoryName: AnsiString; ADeleteExisting, ADoSub,
  ACopyOtherFiles: Boolean; ADoInplaceEdit: Boolean; AProcessFunctions:Boolean): Boolean;
var
  ThisList: TStringList;
  i, InDirLen, FileLen: integer;
  InputDirectoryExpanded, OutputDirRoot, ActualOutPutFile, FileName: AnsiString;
begin
  InputDirectoryExpanded := ExpandFileName(AInputDirName);
  if ADoInplaceEdit then
    ACopyOtherFiles := false
  else
    if AOutputDirectoryName = '' then OutputDirRoot := InputDirectoryExpanded + cAnsiResultDir
    else OutputDirRoot := AOutputDirectoryName;

  InDirLen := Length(InputDirectoryExpanded) + 1;
  ThisList := TStringList.Create;
  Result := false;
  try
    ThisList.Sorted := false;
    ListOfDelphiSourceFilesToProcess(ThisList, InputDirectoryExpanded, ADoSub, ACopyOtherFiles);
    for i := 0 to ThisList.Count - 1 do
    begin
      FileName := ThisList[i];
      FileLen := length(FileName);
      if FileLen > InDirLen then
      begin
        if ADoInplaceEdit then
        begin
          ActualOutPutFile := FileName;
          FileName := NextAvailableBackupName(ActualOutPutFile);
          if not RenameFile(ActualOutPutFile, FileName) then
            raise Exception.Create('Cannot Rename ' + ActualOutPutFile);
        end
        else
          ActualOutPutFile := OutputDirRoot + copy(FileName, InDirLen, Length(FileName));
        ForceDirectories(ExtractFileDir(ActualOutPutFile));
        if ADeleteExisting then DeleteFile(ActualOutPutFile);
        if not FileExists(ActualOutPutFile) then
          if ThisList.Objects[i] = nil then
            ProcessFile(FileName, ActualOutPutFile, AProcessFunctions)
          else
            CopyFile(PAnsiChar(FileName), PAnsiChar(ActualOutPutFile), false);
      end;
    end;
  finally
    ThisList.Free;
  end;
end;

function TAnsiStringProcessObject.ProcessFile(AInputFileName,
  AOutputFileName: AnsiString; AProcessFunctions:Boolean): Boolean;
var
  LocalInputFile: TFileStream;
  CurrentDataPos: Integer;
  OutputData, BitToAdd: AnsiString;
  Tst, LastTst: AnsiChar;
  BufferSz, AddLen, i: Integer;

begin
  Result := false;
  FProcessFunctions:=AProcessFunctions;
  BufferSz := 2000;
  FreeAndNil(FOutputFile);
  LocalInputFile := TFileStream.Create(AInputFileName, fmOpenRead + fmShareDenyNone);
  try
    FFileEnd := LocalInputFile.Seek(0, soFromEnd);
    SetLength(FInputFileData, FFileEnd);
    LocalInputFile.Seek(0, soFromBeginning);
    LocalInputFile.Read(FInputFileData[1], FFileEnd);
  finally
    LocalInputFile.Free;
  end;

  FOutputFile := TFileStream.Create(AOutputFileName, fmCreate);
  try
    SetLength(OutputData, BufferSz + 20);
    FFilePos := 1;
    CurrentDataPos := 1;
    LastTst := ' ';
    while FFilePos <= FFileEnd do
    begin
      Tst := NextData;

      BitToAdd := '';
      if not (LastTst in SetOfNonGapChars) then
      begin

        if Tst in SetOfTestChars + SetOfAltChars then
          BitToAdd := TestFollowingGap(LastTst, Tst)
        else
        If FProcessFunctions then
          BitToAdd:=TestFollowingGap(LastTst,Tst)
         else
          LastTst := Tst;
      end
      else
      begin
        if Tst in SetOfAltChars then
          BitToAdd := TestNoGap(LastTst, Tst)
        else
          LastTst := Tst;
      end;
      AddLen := Length(BitToAdd);
      if AddLen < 1 then
      begin
        OutputData[CurrentDataPos] := Tst;
        Inc(CurrentDataPos);
      end
      else
      begin
        if (CurrentDataPos + AddLen) > BufferSz then
          WriteOutput(OutputData, CurrentDataPos);
        if AddLen > BufferSz then
          Setlength(OutputData, AddLen + 200);
        for i := 1 to AddLen do
        begin
          OutputData[CurrentDataPos] := BitToAdd[i];
          Inc(CurrentDataPos);
        end;
      end;
      if CurrentDataPos >= BufferSz then
        WriteOutput(OutputData, CurrentDataPos);
    end;
    WriteOutput(OutputData, CurrentDataPos);
  finally
    FreeAndNil(FOutputFile);
  end;
end;


function TAnsiStringProcessObject.ProcessOneDirectory(AInputDirName,
  AOutputDirectoryName: AnsiString;
  ADeleteAnyExistingOutput: Boolean;
  ADoInplaceEdit: Boolean;
  AProcessFunctions:Boolean): Boolean;
begin
  Result := false;
  if not DirectoryExists(AInputDirName) then exit;
  if AOutputDirectoryName = '' then
    AOutputDirectoryName := AInputDirName + '\AnsiResult';
  if not DirectoryExists(AOutputDirectoryName) then
    ForceDirectories(AOutputDirectoryName);
  if not ADoInplaceEdit then
    if Lowercase(AOutputDirectoryName) = Lowercase(AInputDirName) then
      raise Exception.Create(
        'Output Directory ' + AOutputDirectoryName + ' is the same as Input ' + AInputDirName);
  Result := ProcessDirectorys(AInputDirName, AOutputDirectoryName,
    ADeleteAnyExistingOutput, false, false, ADoInplaceEdit,AProcessFunctions);
end;

function TAnsiStringProcessObject.ProcessOneFile(AInputFileName,
  AOutputDirectoryName: AnsiString; ADeleteAnyExistingOutput: Boolean; AProcessFunctions:Boolean): Boolean;
var
  OutputFileName: AnsiString;
begin
  Result := false;
  if AInputFileName = '' then exit;
  if AOutputDirectoryName = '' then
    AOutputDirectoryName := ExtractFilePath(AInputFileName) + cAnsiResultDir;
  if not DirectoryExists(AOutputDirectoryName) then
    ForceDirectories(AOutputDirectoryName);
  if not DirectoryExists(AOutputDirectoryName) then
    raise Exception.Create('Output Directory ' + AOutputDirectoryName + ' does not exist');
  if not FileExists(AInputFileName) then
    raise Exception.Create('Input File ' + AInputFileName + ' does not exist');
  OutputFileName := AOutputDirectoryName + '\' + ExtractFileName(AInputFileName);
  if Lowercase(OutputFileName) = Lowercase(AInputFileName) then
    raise Exception.Create(
      'Output File ' + OutputFileName + ' is the same as Input File ' + AInputFileName);
  if ADeleteAnyExistingOutput then
    DeleteFile(OutputFileName);
  if FileExists(OutputFileName) then
    raise Exception.Create('Output File ' + OutputFileName + ' Already exists');
  Result := ProcessFile(AInputFileName, OutputFileName, AProcessFunctions);

end;

procedure TAnsiStringProcessObject.SetUpDataArray;
var
  IniData: TIniFile;
  IniName: string;
  Values: TStringList;
  Sz, i, Loc: Integer;

begin
  try
    Values := TStringList.Create;
    try
      Values.Sorted := true;
      Values.Duplicates := dupIgnore;
      IniName := ChangeFileExt(ParamStr(0), '.Ini');
      IniData := TIniFile.Create(IniName);
      try
        IniData.ReadSections(Values);
        IniData.ReadSectionValues('CONVERSIONS', Values); 
      finally
        IniData.Free;
      end;
      Values.Sort;
      Sz := Values.Count;
      SetLength(FArrayOfFunctions, sz);
      SetLength(FArrayOfFunctionResults, sz);
      for i := 0 to sz - 1 do
      begin
        Loc := pos('=', Values[i]);
        if Loc < 1 then
         Begin
          FArrayOfFunctions[i]:=Uppercase(Trim(Values[i]));
          FArrayOfFunctionResults[i]:=Trim(Values[i])+'A';
         end
        Else
         Begin
          FArrayOfFunctions[i]:=Uppercase(Trim(Copy(Values[i],1,Loc-1)));
          FArrayOfFunctionResults[i]:=Trim(Copy(Values[i],Loc+1,Length(Values[i])));
         End
       end;
      If Values.Count<1 then
        FProcessFunctions:=false;
    finally
      Values.Free;
    end;
  except
    on E: Exception do
      raise Exception.Create('Error in Initialization:: ' + E.Message);
  end;
end;

function TAnsiStringProcessObject.TestFollowingGap(var ALastTst: AnsiChar;
  ATst: AnsiChar): AnsiString;
begin
  case ATst of
    'S', 's': Result := TestForString(ALastTst, ATst);
    'C', 'c': Result := TestForChar(ALastTst, ATst);
    'P', 'p': Result := TestForPChar(ALastTst, ATst);
    '/': Result := TestForSlashComment(ALastTst, ATst);
    '(': Result := TestForBrktStarComment(ALastTst, ATst);
    '{': Result := TestForCrlyBrktComment(ALastTst, ATst);
    '''': Result := TestForQuotedString(ALastTst, ATst);
  else
   if FProcessFunctions and (ATst in SetOfNonGapChars) then
     Result := TestForGeneralString(ALastTst, ATst)
     Else Result:=ATst;
  end; //case
end;

function TAnsiStringProcessObject.TestForBrktStarComment(
  var ARtnLastChar: AnsiChar; ATest: AnsiChar): AnsiString;
var
  Next: AnsiChar;
  Done: boolean;
begin
  Result := ATest;
  if FFilePos + 1 > FFileEnd then Exit;

  Next := NextData;
  ARtnLastChar := Next;
  if Next <> '*' then
  begin
    if Next in SetOfAltChars then
      Result := Result + TestNoGap(ARtnLastChar, Next)
    else
      if Next in SetOfTestChars then
        Result := Result + TestFollowingGap(ARtnLastChar, Next)
      else
        Result := ATest + Next;
 {*****}exit; //*************************************************
  end;



  Result := '(*';
  Done := False;
  while not Done and (FFilePos < FFileEnd) do
  begin
    Next := NextData;
    Result := Result + Next;
    if Next = '*' then
    begin
      Next := NextData;
      Result := Result + Next;
      Done := Next = ')';
    end;
  end;
  ARtnLastChar := Next;
end;

function TAnsiStringProcessObject.TestForChar(var ARtnLastChar: AnsiChar;
  ATest: AnsiChar): AnsiString;
  {sub}function TestForChr(var ASoFar: AnsiString; var ANext: AnsiChar): Boolean;
  begin
    ASoFar := ASoFar + ANext;
    ANext := NextData;
    if not (ANext in SetOfNonGapChars) then
      ASoFar := 'AnsiChr'; //'AnsiChr(Val:Byte):AnsiChar';
    Result := true;
  end;

var
  i: integer;
  Next: AnsiChar;
  Done: boolean;
begin
  Result := ATest;
  if FFilePos + 5 > FFileEnd then Exit;
  Done := False;
  i := 1;
  ARtnLastChar := ATest;
  while not Done do
  begin
    Next := NextData;
    Inc(i);
    case i of
      2: Done := not (Next in ['h', 'H']);
      3: if (Next in ['r', 'R']) then
          Done := TestForChr(Result, Next)
        else
          Done := not (Next in ['a', 'A']);
      4: Done := not (Next in ['r', 'R']);
      5: Done := (Next in SetOfNonGapChars);
    end;
    if Done then
    begin
      if Next in SetOfAltChars then
        Result := Result + TestNoGap(ARtnLastChar, Next)
      else
      begin
        Result := Result + Next;
        ARtnLastChar := Next;
        if (ARtnLastChar in SetOfNonGapChars) then
          Begin
           if FProcessFunctions then
            Result:=TestForGeneralString(ARtnLastChar,Result);
          end
         Else
        if  (Next in SetOfTestChars) then
          raise Exception.Create('Char SetOfTestChars //Cannot get here') //Cannot get here
      end;
    end
    else
      if i > 4 then
      begin
        Done := true;
        Result := 'AnsiChar' + Next;
        ARtnLastChar := Next;
      end
      else
      begin
        Result := Result + Next;
        ARtnLastChar := Next;
      end;
  end;
end;

function TAnsiStringProcessObject.TestForCrlyBrktComment(
  var ARtnLastChar: AnsiChar; ATest: AnsiChar): AnsiString;
var
  Next: AnsiChar;
  Done: boolean;
begin
  Result := ATest;
  Next := '#';
  Done := False;
  while not Done and (FFilePos < FFileEnd) do
  begin
    Next := NextData;
    Result := Result + Next;
    Done := Next = '}';
  end;
  ARtnLastChar := Next;
end;

function TAnsiStringProcessObject.TestForGeneralString(
  var ARtnLastChar: AnsiChar; const ACurrentText: AnsiString): AnsiString;
var
  Tsti, CTL, ArraySz, j: Integer;
  Tst: AnsiChar;
  Terminated: Boolean;
  Nxt: AnsiChar;
  SetOfMatches: set of Byte;
begin
  Nxt:='?';
  if High(FArrayOfFunctions) < 0 then
    SetUpDataArray;
  ArraySz := Length(FArrayOfFunctions);
  Result := '';

  Terminated := False;
  CTL := Length(ACurrentText);
  SetOfMatches := [0..ArraySz-1];
  Tsti := 1;
  while not Terminated and (SetOfMatches <> []) do
  begin
    if Tsti <= CTL then
      Nxt := ACurrentText[Tsti]
    else
      Nxt := NextData;
    Terminated := not (Nxt in SetOfNonGapChars);
    Result := Result + Nxt;

    if not Terminated then
    begin
      if Nxt in ['a'..'z'] then
        Tst := Chr(ord(Nxt) + Ord('A') - Ord('a'))
      else
        Tst := Nxt;
      for j := 0 to ArraySz-1 do
        if (j in SetOfMatches) and
         ((Length(FArrayOfFunctions[j])<Tsti) or (FArrayOfFunctions[j][Tsti] <> Tst)) then
          SetOfMatches := SetOfMatches - [j];
    end;
    inc(Tsti);
  end;
  if Terminated then
  begin
    if SetOfMatches <> [] then
      for j := 0 to ArraySz do
        if (j in SetOfMatches) then
         If (Length(FArrayOfFunctions[j]) = Tsti- 2) then
          Result := FArrayOfFunctionResults[j]+Nxt;

    if Tsti <= CTL then
      While Tsti <= CTL do
        Begin
          Nxt := ACurrentText[Tsti];
          Result:=Result+Nxt;
          Inc(Tsti)
        end;
    If not (Nxt in SetOfNonGapChars) then
      Begin
      Nxt:=NextData;
      Result := Result + TestFollowingGap(ARtnLastChar, Nxt);
      end;
  end
  else
  begin
    if Tsti < CTL+1 then
      Result := ACurrentText
    else
      ARtnLastChar := Nxt;
  end;
end;

function TAnsiStringProcessObject.TestForPChar(var ARtnLastChar: AnsiChar;
  ATest: AnsiChar): AnsiString;

var
  i: integer;
  Next: AnsiChar;
  Done: boolean;
begin
  Result := ATest;
  if FFilePos + 5 > FFileEnd then Exit;
  Done := False;
  i := 1;
  ARtnLastChar := ATest;
  while not Done do
  begin
    Next := NextData;
    Inc(i);
    case i of
      2: Done := not (Next in ['c', 'C']);
      3: Done := not (Next in ['h', 'H']);
      4: Done := not (Next in ['a', 'A']);
      5: Done := not (Next in ['r', 'R']);
      6: Done := (Next in SetOfNonGapChars);
    end;
    if Done then
    begin
      if Next in SetOfAltChars then
        Result := Result + TestNoGap(ARtnLastChar, Next)
      else
      begin
        Result := Result + Next;
        ARtnLastChar := Next;
        if (ARtnLastChar in SetOfNonGapChars) then
          Begin
           if FProcessFunctions then
            Result:=TestForGeneralString(ARtnLastChar,Result);
          end
         Else
        if  (Next in SetOfTestChars) then
          raise Exception.Create('Char SetOfTestChars //Cannot get here') //Cannot get here
      end;
    end
    else
      if i > 5 then
      begin
        Done := true;
        Result := 'PAnsiChar' + Next;
        ARtnLastChar := Next;
      end
      else
      begin
        Result := Result + Next;
        ARtnLastChar := Next;
      end;
  end;
end;

function TAnsiStringProcessObject.TestForQuotedString(
  var ARtnLastChar: AnsiChar; ATest: AnsiChar): AnsiString;
var
  Next: AnsiChar;
  Done: boolean;
begin
  Result := '''';
  if (FFilePos >= FFileEnd) then Exit;

  Next := NextData;
  Result := Result + Next;
  Done := (Next = '''');
  while not Done and (FFilePos < FFileEnd) do
  begin
    Next := NextData;
    Result := Result + Next;
    if Next = '''' then
    begin
      Next := NextData;
      Result := Result + Next;
      Done := Next <> '''';
    end;
  end;
  ARtnLastChar := Next;
end;

function TAnsiStringProcessObject.TestForSlashComment(
  var ARtnLastChar: AnsiChar; ATest: AnsiChar): AnsiString;
var
  Next: AnsiChar;
  Done: boolean;
begin
  Result := ATest;
  if FFilePos + 1 > FFileEnd then Exit;

  Next := NextData;
  ARtnLastChar := ATest;
  if Next <> '/' then
  begin
    if Next in SetOfAltChars then
      Result := Result + TestNoGap(ARtnLastChar, Next)
    else
      Result := ATest + Next;
 {*****}exit; //*************************************************
  end;


  Result := '//';
  Done := False;
  while not Done and (FFilePos < FFileEnd) do
  begin
    Next := NextData;
    Result := Result + Next;
    Done := Next in [#10, #13];
  end;
  ARtnLastChar := Next;
end;

function TAnsiStringProcessObject.TestForString(var ARtnLastChar: AnsiChar;
  ATest: AnsiChar): AnsiString;
var
  i: integer;
  Next: AnsiChar;
  Done: boolean;
begin
  Result := ATest;
  if FFilePos + 6 > FFileEnd then Exit;
  Done := False;
  i := 1;
  ARtnLastChar := ATest;
  while not Done do
  begin
    Next := NextData;
    Inc(i);
    case i of
      2: Done := not (Next in ['t', 'T']);
      3: Done := not (Next in ['r', 'R']);
      4: Done := not (Next in ['i', 'I']);
      5: Done := not (Next in ['n', 'N']);
      6: Done := not (Next in ['g', 'G']);
      7: Done := (Next in SetOfNonGapChars);
    end;
    if Done then
    begin
      if Next in SetOfAltChars then
        Result := Result + TestNoGap(ARtnLastChar, Next)
      else
      begin
        Result := Result + Next;
        ARtnLastChar := Next;
        if (ARtnLastChar in SetOfNonGapChars) then
          Begin
           if FProcessFunctions then
            Result:=TestForGeneralString(ARtnLastChar,Result);
          end
         Else
         If (Next in SetOfTestChars) then
          raise Exception.Create('SetOfTestChars //Cannot get here') //Cannot get here
      end;
    end
    else
      if i > 6 then
      begin
        Done := true;
        if Next = '[' then //String[55]
          Result := Result + Next
        else Result := 'AnsiString' + Next;
        ARtnLastChar := Next;
      end
      else
      begin
        Result := Result + Next;
        ARtnLastChar := Next;
      end;
  end;
end;

function TAnsiStringProcessObject.TestNoGap(var ALastTst: AnsiChar;
  ATst: AnsiChar): AnsiString;
begin
  case ATst of
    '/': Result := TestForSlashComment(ALastTst, ATst);
    '(': Result := TestForBrktStarComment(ALastTst, ATst);
    '{': Result := TestForCrlyBrktComment(ALastTst, ATst);
    '''': Result := TestForQuotedString(ALastTst, ATst);
  end; //case
end;

procedure TAnsiStringProcessObject.WriteOutput(AData: AnsiString;
  var ASizeToWrite: integer);
var
  DN: Int64;
begin
  Dn := 0;
  if FOutputFile <> nil then
    Dn := FOutputFile.Write(AData[1], ASizeToWrite - 1);
  if Dn <> ASizeToWrite - 1 then
    raise Exception.Create('Failed to Write All Data');
  ASizeToWrite := 1;
end;


initialization
finalization
  OnlyProcessObject.Free;
end.

