unit AnsiStringMainForm;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Menus;

type
  TAnsiEditForm = class(TForm)
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    OpenDirectory1: TMenuItem;
    OpenFile1: TMenuItem;
    Process1: TMenuItem;
    CurrentFile1: TMenuItem;
    CurrentDirectory1: TMenuItem;
    IncludeSubDirectories1: TMenuItem;
    LblCurrentDirectory: TLabel;
    LblCurrentFile: TLabel;
    LbxFiles: TListBox;
    SelectFile1: TMenuItem;
    OpenFileDlg: TOpenDialog;
    CbxDeleteExistingDestinationFiles: TCheckBox;
    LblResultDirectory: TLabel;
    DefineOutputDirectory1: TMenuItem;
    N1: TMenuItem;
    CbxCopyOtherSourceFiles: TCheckBox;
    CbxChangeExistingFiles: TCheckBox;
    CbxProcessSystemFunctionCalls: TCheckBox;
    procedure File1Click(Sender: TObject);
    procedure OpenDirectory1Click(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure SelectFile1Click(Sender: TObject);
    procedure OpenFile1Click(Sender: TObject);
    procedure CurrentFile1Click(Sender: TObject);
    procedure DefineOutputDirectory1Click(Sender: TObject);
    procedure CurrentDirectory1Click(Sender: TObject);
    procedure IncludeSubDirectories1Click(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormCreate(Sender: TObject);
    procedure CbxChangeExistingFilesClick(Sender: TObject);
  private
    { Private declarations }
    FCurrentFile: AnsiString;
    FCurrentDirectory, FCurrentResultDir: AnsiString;
    procedure RefreshLabels;
    procedure RefreshListBox;
  public
    { Public declarations }
  end;

var
  AnsiEditForm: TAnsiEditForm;

implementation

uses AnsiStringConvObj, ISDlgOpenDirectory, AnsiStringConfg;

{$R *.dfm}

procedure TAnsiEditForm.CbxChangeExistingFilesClick(Sender: TObject);
begin
 if CbxChangeExistingFiles.Checked then
   begin
     CbxCopyOtherSourceFiles.Checked:=false;
     CbxDeleteExistingDestinationFiles.Checked:=false;
   end;
end;

procedure TAnsiEditForm.CurrentDirectory1Click(Sender: TObject);
begin
  if DirectoryExists(FCurrentDirectory) then
    WorkObj.ProcessOneDirectory(FCurrentDirectory, FCurrentResultDir,
      CbxDeleteExistingDestinationFiles.Checked,CbxChangeExistingFiles.Checked,
      CbxProcessSystemFunctionCalls.Checked);
end;

procedure TAnsiEditForm.CurrentFile1Click(Sender: TObject);
begin
  if FileExists(FCurrentFile) then
    WorkObj.ProcessOneFile(FCurrentFile, FCurrentResultDir, CbxDeleteExistingDestinationFiles.Checked,
    CbxProcessSystemFunctionCalls.Checked);
end;

procedure TAnsiEditForm.DefineOutputDirectory1Click(Sender: TObject);
var
  Dlg: TOpenDirectoryDlg;
begin
  Dlg := TOpenDirectoryDlg.Create(nil);
  try
    Dlg.AllowCreateNew := True;
    if FCurrentResultDir <> '' then
      Dlg.InitialDirectory := FCurrentResultDir
    else
      Dlg.InitialDirectory := FCurrentDirectory;
    if Dlg.Execute then
    begin
      if LowerCase(Dlg.DirectoryName) = LowerCase(FCurrentDirectory) then
        raise Exception.Create('Result Directory Cannot Equal Source Directory');
      FCurrentResultDir := Dlg.DirectoryName;
    end;
  finally
    Dlg.Free;
  end;
  RefreshLabels;
end;

procedure TAnsiEditForm.File1Click(Sender: TObject);
begin
  SelectFile1.Enabled := LbxFiles.Visible and (LbxFiles.Items.count > 0);
end;


procedure TAnsiEditForm.FormActivate(Sender: TObject);
begin
  RefreshLabels;
end;

procedure TAnsiEditForm.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  LastInputDirectory := FCurrentDirectory;
  LastOutputDirectory := FCurrentResultDir;
  SaveSettings(Self);
end;

procedure TAnsiEditForm.FormCreate(Sender: TObject);
begin
  InitialiseApplication;
  if FormHeight > 0 then
  begin
    Top := FormTop;
    Left := FormLeft;
    Height := FormHeight;
    Width := FormWidth;
  end;
  FCurrentDirectory := LastInputDirectory;
  FCurrentResultDir := LastOutputDirectory;
end;

procedure TAnsiEditForm.IncludeSubDirectories1Click(Sender: TObject);
begin
  if DirectoryExists(FCurrentDirectory) then
    WorkObj.ProcessAndCopyDirectoryTree(FCurrentDirectory, FCurrentResultDir,
      CbxDeleteExistingDestinationFiles.Checked, CbxCopyOtherSourceFiles.Checked,
      CbxChangeExistingFiles.Checked,CbxProcessSystemFunctionCalls.Checked);
end;

procedure TAnsiEditForm.OpenDirectory1Click(Sender: TObject);
var
  Dlg: TOpenDirectoryDlg;
begin
  Dlg := TOpenDirectoryDlg.Create(nil);
  try
    Dlg.InitialDirectory := FCurrentDirectory;
    Dlg.AllowCreateNew := false;
    if Dlg.Execute then
    begin
      FCurrentDirectory := Dlg.DirectoryName;
    end;
  finally
    Dlg.Free;
  end;
  RefreshLabels;
  RefreshListBox;
end;

procedure TAnsiEditForm.OpenFile1Click(Sender: TObject);
var
  ResultDir: string;
  //  PChar  Chr  Test a String in a Comment  Char PChr
  tt: PChar;
  ttt: Char;
  (*
  PChar  Chr
  Test a String in a Comment
  Char PChr
  *)

  //  PChar  Chr  Test a String in a Comment  Char PChr

{  tt:PChar;
  ttt:Char;
 }
const
  vvv = 'Use Some String lllll Char PChr';
begin
  ttt := Chr(56);
  tt := @ttt;
  ttt := tt^;
  if FCurrentDirectory <> '' then
    OpenFileDlg.InitialDir := FCurrentDirectory;
  if FCurrentDirectory <> '' then
   OutputDebugString('Test');
  tt:=StrAlloc(5);
  StrDispose(tt);
  ResultDir := FCurrentResultDir;
  if OpenFileDlg.Execute then
  begin
    FCurrentFile := OpenFileDlg.FileName;
    FCurrentDirectory := ExtractFileDir(FCurrentFile);
    RefreshLabels;
    RefreshListBox;
  end;
end;


procedure TAnsiEditForm.RefreshLabels;
begin
  LblCurrentDirectory.Caption := 'Process Directory::' + FCurrentDirectory;
  LblCurrentFile.Caption := 'Current File::' + FCurrentFile;
  LblResultDirectory.Caption := 'Output to Directory::' + FCurrentResultDir;
end;


procedure TAnsiEditForm.RefreshListBox;
var
  Lst: TStringList;
begin
  LbxFiles.Clear;
  if FCurrentDirectory <> '' then
  begin
    Lst := TStringList.Create;
    try
      Lst.Sorted := true;
      Lst.Duplicates := dupIgnore;
      WorkObj.ListOfDelphiSourceFilesToProcess(Lst, FCurrentDirectory, false, false);
      LbxFiles.Items.AddStrings(Lst);
    finally
      Lst.Free;
    end;
  end;
  LbxFiles.Visible := LbxFiles.Items.Count > 0;
end;

procedure TAnsiEditForm.SelectFile1Click(Sender: TObject);
var
  i: integer;
begin
  if not LbxFiles.Visible then exit;

  i := LbxFiles.ItemIndex;
  if i < 0 then exit;

  FCurrentFile := LbxFiles.Items[i];
  RefreshLabels;
end;







end.

